@@ -10,6 +10,7 @@ __all__ = [
 
 import enum
 import datetime
+import pathlib
 import typing
 from dataclasses import dataclass
 
@@ -77,4 +78,7 @@ class ShellReturn:
 @dataclass
 class AdbDeviceInfo:
     serial: str
-    state: str
\ No newline at end of file
+    state: str
+
+
+StrOrPathLike = typing.Union[str, pathlib.Path]
\ No newline at end of file
