@@ -5,6 +5,7 @@ import socket
 import subprocess
 import sys
 import tempfile
+import threading
 import time
 import typing
 import zipfile
@@ -179,3 +180,30 @@ class APKReader:
         print("main-activity:", am.main_activity)
         print("version-name:", am.version_name)
         print("version-code:", am.version_code)
+
+
+class StopEvent:
+    def __init__(self):
+        self.__stop = threading.Event()
+        self.__done = threading.Event()
+
+    def stop(self):
+        """ send stop signal """
+        self.__stop.set()
+    
+    def stop_wait(self, timeout=None):
+        """ send stop signal and wait signal accepted """
+        self.__stop.set()
+        if not self.__done.wait(timeout):
+            raise TimeoutError("wait for stopped timeout", timeout)
+
+    def is_stopped(self) -> bool:
+        return self.__done.is_set()
+
+    def done(self):
+        """ for worker thread to notify stop signal accepted """
+        self.__done.set()
+    
+    def reset(self):
+        self.__stop.clear()
+        self.__done.clear()
\ No newline at end of file
